# Script to analyze the Cernuda SAJT experiment, designed by Claudia Paneda.
# This script works on data gathered using Ibex Farm and pre-processed with R.
# Files required for the script: (1) file with acceptability judgments; (2) file with working-memory scores.
# Created by Sol Lago and Claudia Paneda, December 2016.

# Island conditions (n=32, items 1-32). 
# Example from the subject constructions:
# non-island/short: Who _ believes [that the discourse offended Julia so much yesterday]?
# non-island/long:  Who do you believe [that _ offended Julia so much with the discourse]?
# island/short:     Who _ believes [that the discourse of the director offended Julia]?
# island/long:      Of who do you believe [that the discourse _ offended Julia so much]?

# Construction types.
# Whether islands:            items 1-8. 
# Complex NP islands (CNCP):  items 9-16.
# Subject islands:            items 17-24.
# Adjunct islands:            items 25-32.

# Fillers (n = 48; items 33-80).
# Grammatical:    "filler-grammatical"
# Ungrammatical:  "filler-ungrammatical"

# The .csv input file has the following columns:
# Participant: participant label.
# Item: item number.
# Type: either 'practice', a letter indicating condition or 'fXX'.
# Order: item position in the entire experiment.
# Response: what participants selected: yes for "acceptable" and no for "not acceptable".
# RT: reaction time to the judgement
# Other demographic variables.

# Set working directory.
# Load packages -----------------------------------------------------------
library(binom)                              # binomial confidence intervals
library(Hmisc)                              # error bars 
library(tidyr)                              # data frame restructuring
library(plyr)                               # data frame computations                           
library(ggplot2)                            # plots
library(scales)                             # customize axes scales in plots
library(grid)                               # concatenate ggplots
library(rstan)                              # bayesian analysis
library(brms)                               # bayesian analysis
library(bayesplot)                          # bayesian analysis
library(extrafont)                          # Fonts for plots
library(ggpubr)                             # Combining plots
library(ggrepel)
library(RColorBrewer)
library(gridExtra)

# Set up for Bayesian analyses ==================================================================================================
# Save compiled models
rstan_options(auto_write = TRUE)

# Parallelize the chains using all the cores.
# If we want to have an extra core free:options(mc.cores = parallel::detectCores() - 1)
options(mc.cores = parallel::detectCores())


# Define global variables ==================================================================================================

OUT <- c(0,2000)                            # RT outlier trimming cutoffs
POI <- c('Distancelong:Structureisland')    # statistical parameters of interest


# Define functions ==================================================================================================

# Center numeric predictors.
c. <- function(x){scale(as.numeric(x), scale=F)}

# Calculate normal standard error ( = standard deviation / sqrt(n)).
se <- function(x) {return(sqrt(var(x, na.rm=TRUE)/length(na.omit(x))))}

# Calculate binomial standard error. The standard deviation of a binomial variable is
#x is coded as 1 (success) or 0 (failure), so number of successes is the sum of x = 1.
# sigma = sqrt(k*p*q/n), where q = 1-p and k = number of bernoulli random variables = 1, so
# SE = sigma / sqrt(n) = sqrt(k*p*q) / sqrt(n) = sqrt(p*q/n).
se.bin <- function(x){
  n.success = sum(x)  
  n         = length(na.omit(x))
  p         = n.success / n
  return(sqrt(p*(1-p)/n))
}

# Trim RTs.
# The function returns the number of trimmed observations by-participant.
# The function takes four arguments:
# df:  to-be-trimmed data frame.
# variable: the name of the grouping variable (between quotation marks).
# min: smaller allowed RT.
# max: larger allowed RT.
trim <- function(df, variable, min, max) {
  
  # Store table with untrimmed counts of observations per region.
  ntotal <- table(df[ , variable])
  
  # Trim RTs below 'min', above 'max' and NA trials.
  df_trimmed <- subset(df, RT > min & RT < max & !is.na(Response))
  
  # Compute and print exclusion percentages by participant.
  ntrimmed <- table(df_trimmed[ , variable])
  npercent <- (ntotal - ntrimmed) / ntotal * 100
  print(list('Exclusion percentages', npercent))
  
  return(df_trimmed)
}

# Compare LOOIC/WAIC values (from elpd values).
# The model in the first row of lc is the model with the largest exact expected log predictive density (ELPD).
compareIC <- function(..., criterion='loo'){
  lc     <- loo_compare(..., criterion=criterion)
  ICdiff <- lc[-1, 'elpd_diff', drop=F]*-2
  sediff <- lc[-1, 'se_diff', drop=F]*2
  output <- data.frame(ICdiff = as.vector(ICdiff), SE_diff = sediff, IC_t = as.vector(ICdiff/sediff))
  cat(paste0('\n', rownames(lc)[1], ' has the smallest IC.', '\n\n'))
  return(round(output, 1))
}

# Extracting the legend from one plot and later pasting it into another plot.
g_legend<-function(a.gplot){
  tmp <- ggplot_gtable(ggplot_build(a.gplot))
  leg <- which(sapply(tmp$grobs, function(x) x$name) == "guide-box")
  legend <- tmp$grobs[[leg]]
  return(legend)}


# Using two decimals in plots.
scaleFUN <- function(x) sprintf("%.1f", x)



# Plotting parameters -----------------------------------------------------
col         <- c('black', 'mediumvioletred') 
dodge       <- position_dodge(.2)     
facet_names <- c(`subject`='Subject', `complex NP`='Complex NP', `adjunct`='Adjunct', `interrogative`='Interrogative')    
loadfonts(device = "win")
# Read in and prepare data ------------------------------------------------

data     <- read.csv('islands_data.csv', quote="'", header=T, fileEncoding = 'latin1') # Acceptability judgment results.
names(data)[names(data) == "..Participant"] <- "Participant"
wmscores <- read.csv('WMScores.csv', quote="'", header=T, fileEncoding = 'UTF-8') # Working memory scores.

# Merge experimental information with WM scores.
data <- merge(data, wmscores, by = c('Participant'), all.x=TRUE)
write.table(data, file= 'islands_Spanish_data.txt', quote=F, row.names=F)  

# Factorize 'Participant', 'Item' and 'Order".
data$Participant <- factor(data$Participant)
data$Item        <- factor(data$Item)
data$Order       <- factor(data$Order)

# Exclude subjects who reported reading impairments.
data <- droplevels(subset(data, Impairment == 'none'))

# Make responses numeric ('accepted' = 1, 'rejected' = 0).
data$RespNum <- ifelse(data$Response == 'accepted', 1, 0)

# Exclude participants with missing trials > 20% in the fillers, suspected impairments 
# and chance performance in verifying the operations of the WM score.
# One additional participant was deleted directly from the source file, as their data was not fully recorded.
data <- droplevels(data[!(data$Participant %in% c('h6b5e2', 'n3j5m', 'j5n5t', 'c6z55')), ])

# Code for target response and accuracy.
data$TargetResponse <- ifelse(data$Condition %in% c('filler_ungrammatical', 'long_island', 'long_ungrammatical', 'short_ungrammatical'), 0, 1)
data$Accuracy       <- ifelse(data$RespNum == data$TargetResponse, 1, 0)


# Process demographic information -------------------------------------------------------------

# Create data frame with only demographic information.
demographic <- subset(data, select=-c(Condition, Item, Order, Response, RT, RespNum, TargetResponse, Accuracy))  
demographic <- droplevels(subset(demographic, !duplicated(Participant)))

# Print demographic summary.
print(summary(demographic))

# Show how many participants reported speaking which language combinations.
table(demographic$OtherLang)


# <<<< Calculate and plot accuracy in filler items >>>>

# Fillers -----------------------------------------------------------------

# Get only filler items.
filler <- droplevels(subset(data, Condition %in% c('filler_grammatical', 'filler_ungrammatical')))

# Remove RTs outside bounds and NA responses and print exclusion percentages by participants.
filler <- trim(filler, 'Participant', OUT[1], OUT[2])

# Compute accuracy in the fillers by participant.
filler.avg <- ddply(filler, .(Participant), summarise, 
                    N            = length(Accuracy),
                    MeanAccuracy = mean(Accuracy)*100,
                    Lci          = as.numeric(binom.confint(sum(Accuracy), length(Accuracy), methods='agresti-coull')[5])*100,
                    Uci          = as.numeric(binom.confint(sum(Accuracy), length(Accuracy), methods='agresti-coull')[6])*100,
                    RTcorrect    = mean(RT[Accuracy==1]),
                    SEcorrect    = se(RT[Accuracy==1]))

print(summary(filler.avg))


# Process island items -------------------------------------------------------------

# Get only island trials
island <- droplevels(subset(data, Condition %in% c('long_island', 'short_island', 'long_nonisland', 'short_nonisland')))

# Code for factors.
island$Distance     <- with(island, ifelse(island$Condition %in% c('long_island','long_nonisland'), 'long', 'short'))
island$Structure    <- with(island, ifelse(Condition %in% c('long_island', 'short_island'), 'island', 'non-island'))
island$Construction <- with(island, ifelse(Item %in% 1:8, 'interrogative', ifelse(Item %in% 9:16, 'complex NP', ifelse(Item %in% 17:24, 'subject', 'adjunct'))))

# Reorder factor levels.
island$Condition    <- factor(island$Condition, levels=c('short_nonisland', 'short_island', 'long_nonisland', 'long_island'))
island$Distance     <- factor(island$Distance, levels = c('short', 'long'))
island$Structure    <- factor(island$Structure, levels = c('non-island', 'island'))
island$Construction <- factor(island$Construction, levels = c('subject','complex NP', 'adjunct', 'interrogative'))

# Remove RTs outside bounds and NA responses and print exclusion percentages.
island <- trim(island, 'Condition', OUT[1], OUT[2])

# Compute proportion of accepted items and RTs by condition. 
island_avg <- ddply(island, .(Condition, Distance, Structure), summarise, 
                    N          = length(RespNum),
                    Mean_Acc   = mean(RespNum),
                    sd_Acc        = sd(RespNum),
                    SE_AccProp = se.bin(RespNum),
                    Lci        = as.numeric(binom.confint(sum(RespNum), length(RespNum), methods='agresti-coull')[5]),
                    Uci        = as.numeric(binom.confint(sum(RespNum), length(RespNum), methods='agresti-coull')[6]),
                    Mean_RT    = mean(RT[Accuracy==1]),
                    SE_RT      = sd(RT[Accuracy==1]))

island_type <- ddply(island, .(Construction, Condition, Distance, Structure), summarise, 
                     N             = length(RespNum),
                     Mean_Acc      = mean(RespNum),
                     sd_Acc        = sd(RespNum),
                     SE_AccProp    = se.bin(RespNum),
                     Lci           = as.numeric(binom.confint(sum(RespNum), length(RespNum), methods='agresti-coull')[5]),
                     Uci           = as.numeric(binom.confint(sum(RespNum), length(RespNum), methods='agresti-coull')[6]),
                     Mean_RT       = mean(RT[Accuracy==1]),
                     SE_RT         = sd(RT[Accuracy==1]))


# Plot interactions by island type (Figure 2) -----------------------------
jpeg('Figure2.jpeg', units='cm', width=16, height=16, res=300)
ggplot(island_type, aes(x = Distance, y = qlogis(Mean_Acc), group = Structure, colour = Structure)) +
  facet_wrap(. ~ Construction, labeller = as_labeller(facet_names)) + 
  geom_line(aes(linetype = Structure), size = 1, position = dodge) +
  geom_errorbar(aes(ymin = qlogis(Lci), ymax = qlogis(Uci)), width = .15, size = 1, position = dodge, show.legend = FALSE) +
  geom_point(size = 3, position = dodge, show.legend = FALSE) +
  labs(x = NULL, y = 'Acceptability [log odds]') +
  scale_colour_manual(values = col) +
  scale_x_discrete(labels = c('short distance', 'long distance')) +
  theme_light() +
  theme(text = element_text(size = 15, family = 'Arial', colour = 'gray28'), legend.position = 'top',  axis.title.y=element_text(size=13))
dev.off()

# DD scores ---------------------------------------------------------------

# Compute acceptability percentages by participant.
DD_island <- ddply(island, .(Participant, Condition, WMPartialCreditUnit, WMAllOrNothingUnit, WMPartialCreditLoad, WMAllOrNothingLoad), summarise, 
                   AcceProp = mean(RespNum)*100)

# Create separate columns for the four experimental conditions and calculate DD scores.
DD_island <- DD_island %>% 
  spread(Condition, AcceProp) %>%
  mutate(DDscore = (long_island - short_island) - (long_nonisland - short_nonisland))

# Plot DD scores as a function of WMPartialCreditUnit (all construction types collapsed).
# The line is calculated through linear regression on the by-participant DDscores.
print(ggplot(DD_island, aes(x = WMPartialCreditUnit, y = DDscore)) +
  geom_smooth(method = lm, colour = col[2], fill = col[2]) +
  geom_point(size = 1) +
  labs(x = 'Working memory score', y = 'DD score') +
  scale_x_continuous(limits = c(.25,.95), breaks = seq(0,1,.1)) +
  theme_bw() +
  theme(text = element_text(size = 20, family = 'Arial', colour = 'gray28'))) 


# <<<< Statistics: priors >>>>

# Priors ------------------------------------------------------------------

# Prior without random effects.
priors        <- c(prior('normal(0,10)', class = 'Intercept'),
                   set_prior('normal(0,10)', class = 'b'),
                   set_prior('normal(0,10)', class = 'sd'))

# Prior with random effects.
priors.slopes <- c(prior('normal(0,10)', class = 'Intercept'),
                   set_prior('normal(0,10)', class = 'b'),
                   set_prior('normal(0,10)', class = 'sd'),
                   set_prior('lkj(2)', class = 'cor'))

# Statistics: model selection for random effects  --------------------------

# Iterations are increased to 10000 to make model comparisons more stable.
# Sum contrasts are used to estimate the effects across construction types.

# Contrasts.
contrasts(island$Distance)     <- contr.treatment(levels(island$Distance), base=which(levels(island$Distance) == 'short'))
contrasts(island$Structure)    <- contr.treatment(levels(island$Structure), base=which(levels(island$Structure) == 'non-island'))
contrasts(island$Construction) <- contr.sum(4)/2 # Average of all 4 island types

# Assess inclusion of by-participant slopes -------------------------------

# Intercept-only model vs. models with by-participant slopes (distance, structure, construction).
m_sum    <- brm(RespNum ~ (1|Participant) + (1|Item) 
                + Distance * Structure * Construction 
                + Distance * Structure * c.(WMPartialCreditUnit),
                prior = priors,
                family = bernoulli(),
                iter = 10000, 
                warmup = 2000,
                data = island)

m_sum_ds <- brm(RespNum ~ (1+Distance|Participant) + (1|Item) 
                + Distance * Structure * Construction 
                + Distance * Structure * c.(WMPartialCreditUnit),
                prior = priors.slopes,
                family = bernoulli(),
                iter = 10000, 
                warmup = 2000,
                data = island)

m_sum_ss <- brm(RespNum ~ (1+Structure|Participant) + (1|Item)  
                + Distance * Structure * Construction 
                + Distance * Structure * c.(WMPartialCreditUnit),
                prior = priors.slopes,
                family = bernoulli(),
                iter = 10000, 
                warmup = 2000,
                data = island)

m_sum_ts <- brm(RespNum ~ (1+Construction|Participant) + (1|Item) 
                + Distance * Structure * Construction 
                + Distance * Structure * c.(WMPartialCreditUnit),
                prior = priors.slopes,
                family = bernoulli(),
                iter = 10000, 
                warmup = 2000,
                data = island)

# Inspect models.
summary(m_sum)
summary(m_sum_ds)
summary(m_sum_ss)
summary(m_sum_ts)

# Do model comparisons.
m_sum    <- add_criterion(m_sum, 'loo', reloo = T)
m_sum_ds <- add_criterion(m_sum_ds, 'loo', reloo = T)
m_sum_ss <- add_criterion(m_sum_ss, 'loo', reloo = T)
m_sum_ts <- add_criterion(m_sum_ts, 'loo', reloo = T)

compareIC(m_sum, m_sum_ds)
compareIC(m_sum, m_sum_ss)
compareIC(m_sum, m_sum_ts)


# Assess inclusion of by-item slopes --------------------------------------
# Intercept-only model vs. models with by-item slopes (distance, structure, working memory).
m_sum_di  <- brm(RespNum ~ (1|Participant) + (1+Distance|Item) 
                 + Distance * Structure * Construction 
                 + Distance * Structure * c.(WMPartialCreditUnit),
                 prior = priors.slopes,
                 family = bernoulli(),
                 iter = 10000, 
                 warmup = 2000,
                 data = island)

m_sum_si  <- brm(RespNum ~ (1|Participant) + (1+Structure|Item) 
                 + Distance * Structure * Construction 
                 + Distance * Structure * c.(WMPartialCreditUnit),
                 prior = priors.slopes,
                 family = bernoulli(),
                 iter = 10000, 
                 warmup = 2000,
                 data = island)

m_sum_mi  <- brm(RespNum ~ (1|Participant) + (1+c.(WMPartialCreditUnit)|Item) 
                 + Distance * Structure * Construction 
                 + Distance * Structure * c.(WMPartialCreditUnit),
                 prior = priors.slopes,
                 family = bernoulli(),
                 iter = 10000, 
                 warmup = 2000,
                 data = island)

# Inspect models.
summary(m_sum_di)
summary(m_sum_si)
summary(m_sum_mi)

# Do model comparisons.
m_sum_di <- add_criterion(m_sum_di, 'loo', reloo = T)
m_sum_si <- add_criterion(m_sum_si, 'loo', reloo = T)
m_sum_mi <- add_criterion(m_sum_mi, 'loo', reloo = T)

compareIC(m_sum, m_sum_di)
compareIC(m_sum, m_sum_si)
compareIC(m_sum, m_sum_mi)

# Assess inclusion of 3-way interaction with Construction -----------------

# Compute model without 3-way interaction with type.
m_notype <- update(m_sum, . ~ . -Distance:Structure:Construction)
summary(m_notype)

# Do model comparison.
m_notype <- add_criterion(m_notype, 'loo', reloo = T)
compareIC(m_sum, m_notype)

# Assess inclusion of 3-way interaction with Working Memory ---------------

# Compute model without 3-way interaction with working memory.
m_nowm <- update(m_sum, . ~ . -Distance:Structure:c.(WMPartialCreditUnit))
summary(m_nowm)

# Do model comparison.
m_nowm <- add_criterion(m_nowm, 'loo', reloo = T)
compareIC(m_sum, m_nowm)

# Statistics: final models ------------------------------------------------
# Overall model
# Baseline model was already calculated above (m_sum).
# Plot model parameters and chains.
plot(m_sum, pars = '^b_')

# Do predictive checks.
pp_check(m_sum)

# Calculate model with reversed contrasts.
contrasts(island$Distance)     <- contr.treatment(levels(island$Distance), base=which(levels(island$Distance) == 'long'))
contrasts(island$Structure)    <- contr.treatment(levels(island$Structure), base=which(levels(island$Structure) == 'island'))
contrasts(island$Construction) <- contr.sum(4)/2

m_sum_rev <- brm(formula = RespNum ~ Distance * Structure * c.(WMPartialCreditUnit)
             + (1 | Participant) 
             + (1 | Item),
             prior = priors,
             family = bernoulli(),
             iter = 10000, 
             warmup = 2000,
             data = island)

# Save models.
saveRDS(m_sum, 'm_all.rds')
saveRDS(m_sum_rev, 'm_all_rev.rds')

# Subject islands ---------------------------------------------------------

# Baseline contrasts.
contrasts(island$Construction) <- contr.treatment(levels(island$Construction), base=which(levels(island$Construction) == 'subject'))
contrasts(island$Distance)     <- contr.treatment(levels(island$Distance), base=which(levels(island$Distance) == 'short'))
contrasts(island$Structure)    <- contr.treatment(levels(island$Structure), base=which(levels(island$Structure) == 'non-island'))

m <- brm(formula = RespNum ~ Distance * Structure * Construction + Distance * Structure * c.(WMPartialCreditUnit)
         + (1 | Participant) 
         + (1 | Item),
         prior = priors,
         family = bernoulli(),
         data = island)

# Plot model parameters and chains.
plot(m, pars = '^b_')

# Do predictive checks.
pp_check(m)

# Plot distance*structure interaction.
# Plot basic density plot, get y-coordinates for lines and shaded areas and 
# calculate maximum height of the vertical line that shows the posterior mean.
b             <- fixef(m) [POI, ]
pdata         <- data.frame(Construction='Subject', posterior_samples(m, pars='b_Distancelong:Structureisland', exact_match=TRUE))
p             <- ggplot(pdata, aes(x = b_Distancelong.Structureisland)) + geom_density(size = .8)
d             <- ggplot_build(p)$data[[1]]
dd            <- approxfun(d$x, d$y)
pmean_subject <- b[['Estimate']]
lmax_subject  <- dd(pmean_subject)

plot_subject <- 
  ggplot(pdata, aes(x = b_Distancelong.Structureisland)) + 
  geom_segment(aes(x = pmean_subject, y = 0, xend = pmean_subject, yend = lmax_subject), colour = col[2], size = .5) +
  geom_density(size = .8) +
  geom_area(data = subset(d, x > b[['Q2.5']] & x < b[['Q97.5']]), aes(x = x, y = y), fill = col[2], alpha = .3) +
  geom_vline(xintercept=0, colour='gray', size=.8, lty='dashed') +
  scale_x_continuous(limits=c(-9, 2), breaks=seq(-18, 2, 2)) +
  facet_wrap(. ~ Construction) + 
  theme_light() +
  theme(text=element_text(size = 16, family = 'Arial', colour = 'gray28'), axis.text.y = element_blank(), axis.title = element_blank())

# Calculate model with reversed contrasts.
contrasts(island$Construction) <- contr.treatment(levels(island$Construction), base=which(levels(island$Construction) == 'subject'))
contrasts(island$Distance)     <- contr.treatment(levels(island$Distance), base=which(levels(island$Distance) == 'long'))
contrasts(island$Structure)    <- contr.treatment(levels(island$Structure), base=which(levels(island$Structure) == 'island'))

m_rev<- brm(formula = RespNum ~ Distance * Structure * Construction + Distance * Structure * c.(WMPartialCreditUnit)
            + (1 | Participant) 
            + (1 | Item),
            prior = priors,
            family = bernoulli(),
            data = island)

# Save models.
saveRDS(m, 'm_subject.rds')
saveRDS(m_rev, 'm_subject_rev.rds')

# Complex NP islands ------------------------------------------------------

# Baseline contrasts.
contrasts(island$Construction) <- contr.treatment(levels(island$Construction), base=which(levels(island$Construction) == 'complex NP'))
contrasts(island$Distance)     <- contr.treatment(levels(island$Distance), base=which(levels(island$Distance) == 'short'))
contrasts(island$Structure)    <- contr.treatment(levels(island$Structure), base=which(levels(island$Structure) == 'non-island'))

m <- brm(formula = RespNum ~ Distance * Structure * Construction + Distance * Structure * c.(WMPartialCreditUnit)
         + (1 | Participant) 
         + (1 | Item),
         prior = priors,
         family = bernoulli(),
         data = island)

# Plot model parameters and chains.
plot(m, pars = '^b_')

# Do predictive checks.
pp_check(m)

# Plot distance*structure interaction.
# Plot basic density plot, get y-coordinates for lines and shaded areas and 
# calculate maximum height of the vertical line that shows the posterior mean.
b        <- fixef(m) [POI, ]
pdata    <- data.frame(Construction='Complex NP', posterior_samples(m, pars='b_Distancelong:Structureisland', exact_match=TRUE))
p        <- ggplot(pdata, aes(x = b_Distancelong.Structureisland)) + geom_density(size = .8)
d        <- ggplot_build(p)$data[[1]]
dd       <- approxfun(d$x, d$y)
pmean_np <- b[['Estimate']]
lmax_np  <- dd(pmean_np)

plot_np <-  
  ggplot(pdata, aes(x = b_Distancelong.Structureisland)) + 
  geom_segment(aes(x = pmean_np, y = 0, xend = pmean_np, yend = lmax_np), colour = col[2], size = .5) +
  geom_density(size = .8) +
  geom_area(data = subset(d, x > b[['Q2.5']] & x < b[['Q97.5']]), aes(x = x, y = y), fill = col[2], alpha = .3) +
  geom_vline(xintercept=0, colour='gray', size=.8, lty='dashed') +
  scale_x_continuous(limits=c(-9, 2), breaks=seq(-8, 2, 2)) +
  facet_wrap(. ~ Construction) + 
  theme_light() +
  theme(text=element_text(size = 16, family = 'Arial', colour = 'gray28'), axis.text.y = element_blank(), axis.title = element_blank())

# Calculate model with reversed contrasts.
contrasts(island$Construction) <- contr.treatment(levels(island$Construction), base=which(levels(island$Construction) == 'complex NP'))
contrasts(island$Distance)     <- contr.treatment(levels(island$Distance), base=which(levels(island$Distance) == 'long'))
contrasts(island$Structure)    <- contr.treatment(levels(island$Structure), base=which(levels(island$Structure) == 'island'))

m_rev<- brm(formula = RespNum ~ Distance * Structure * Construction + Distance * Structure * c.(WMPartialCreditUnit)
            + (1 | Participant) 
            + (1 | Item),
            prior = priors,
            family = bernoulli(),
            data = island)

# Save models.
saveRDS(m, 'm_np.rds')
saveRDS(m_rev, 'm_np_rev.rds')


# Adjunct islands ---------------------------------------------------------

# Baseline contrasts.
contrasts(island$Construction) <- contr.treatment(levels(island$Construction), base=which(levels(island$Construction) == 'adjunct'))
contrasts(island$Distance)     <- contr.treatment(levels(island$Distance), base=which(levels(island$Distance) == 'short'))
contrasts(island$Structure)    <- contr.treatment(levels(island$Structure), base=which(levels(island$Structure) == 'non-island'))

m <- brm(formula = RespNum ~ Distance * Structure * Construction + Distance * Structure * c.(WMPartialCreditUnit)
         + (1 | Participant) 
         + (1 | Item),
         prior = priors,
         family = bernoulli(),
         data = island)

# Plot model parameters and chains.
plot(m, pars = '^b_')

# Do predictive checks.
pp_check(m)

# Plot distance*structure interaction.
# Plot basic density plot, get y-coordinates for lines and shaded areas and 
# calculate maximum height of the vertical line that shows the posterior mean.
b             <- fixef(m) [POI, ]
pdata         <- data.frame(Construction='Adjunct', posterior_samples(m, pars='b_Distancelong:Structureisland', exact_match=TRUE))
p             <- ggplot(pdata, aes(x = b_Distancelong.Structureisland)) + geom_density(size = .8)
d             <- ggplot_build(p)$data[[1]]
dd            <- approxfun(d$x, d$y)
pmean_adjunct <- b[['Estimate']]
lmax_adjunct  <- dd(pmean_adjunct)

plot_adjunct <-  
  ggplot(pdata, aes(x = b_Distancelong.Structureisland)) + 
  geom_segment(aes(x = pmean_adjunct, y = 0, xend = pmean_adjunct, yend = lmax_adjunct), colour = col[2], size = .5) +
  geom_density(size = .8) +
  geom_area(data = subset(d, x > b[['Q2.5']] & x < b[['Q97.5']]), aes(x = x, y = y), fill = col[2], alpha = .3) +
  geom_vline(xintercept=0, colour='gray', size=.8, lty='dashed') +
  scale_x_continuous(limits=c(-9, 2), breaks=seq(-8, 2, 2)) +
  facet_wrap(. ~ Construction) + 
  labs(x = 'structure and distance interaction', y = NULL) +
  theme_light() +
  theme(text=element_text(size = 16, family = 'Arial', colour = 'gray28'), axis.text.y = element_blank()) +
  theme(axis.title.x = element_text(size = 14))

# Calculate model with reversed contrasts.
contrasts(island$Construction) <- contr.treatment(levels(island$Construction), base=which(levels(island$Construction) == 'adjunct'))
contrasts(island$Distance)     <- contr.treatment(levels(island$Distance), base=which(levels(island$Distance) == 'long'))
contrasts(island$Structure)    <- contr.treatment(levels(island$Structure), base=which(levels(island$Structure) == 'island'))

m_rev <- brm(formula = RespNum ~ Distance * Structure * Construction + Distance * Structure * c.(WMPartialCreditUnit)
             + (1 | Participant) 
             + (1 | Item),
             prior = priors,
             family = bernoulli(),
             data = island)

# Save models.
saveRDS(m, 'm_adjunct.rds')
saveRDS(m_rev, 'm_adjunct_rev.rds')


# Interrogative islands ---------------------------------------------------

# Baseline contrasts.
contrasts(island$Construction) <- contr.treatment(levels(island$Construction), base=which(levels(island$Construction) == 'interrogative'))
contrasts(island$Distance)     <- contr.treatment(levels(island$Distance), base=which(levels(island$Distance) == 'short'))
contrasts(island$Structure)    <- contr.treatment(levels(island$Structure), base=which(levels(island$Structure) == 'non-island'))

m <- brm(formula = RespNum ~ Distance * Structure * Construction + Distance * Structure * c.(WMPartialCreditUnit)
         + (1 | Participant) 
         + (1 | Item),
         prior = priors,
         family = bernoulli(),
         data = island)

# Plot model parameters and chains.
plot(m, pars = '^b_')

# Do predictive checks.
pp_check(m)

# Plot distance*structure interaction.
# Plot basic density plot, get y-coordinates for lines and shaded areas and 
# calculate maximum height of the vertical line that shows the posterior mean.
b                   <- fixef(m) [POI, ]
pdata               <- data.frame(Construction='Interrogative', posterior_samples(m, pars='b_Distancelong:Structureisland', exact_match=TRUE))
p                   <- ggplot(pdata, aes(x = b_Distancelong.Structureisland)) + geom_density(size = .8)
d                   <- ggplot_build(p)$data[[1]]
dd                  <- approxfun(d$x, d$y)
pmean_interrogative <- b[['Estimate']]
lmax_interrogative  <- dd(pmean_interrogative)

plot_interrogative <- 
  ggplot(pdata, aes(x = b_Distancelong.Structureisland)) + 
  geom_segment(aes(x = pmean_interrogative, y = 0, xend = pmean_interrogative, yend = lmax_interrogative), colour = col[2], size = .5) +
  geom_density(size = .8) +
  geom_area(data = subset(d, x > b[['Q2.5']] & x < b[['Q97.5']]), aes(x = x, y = y), fill = col[2], alpha = .3) +
  geom_vline(xintercept=0, colour='gray', size=.8, lty='dashed') +
  scale_x_continuous(limits=c(-9, 2), breaks=seq(-8, 2, 2)) +
  facet_wrap(. ~ Construction) + 
  labs(x = 'structure and distance interaction', y = NULL) +
  theme_light() +
  theme(text=element_text(size = 16, family = 'Arial', colour = 'gray28'), axis.text.y = element_blank()) +
  theme(axis.title.x = element_text(size = 14))

# Calculate model with reversed contrasts.
contrasts(island$Construction) <- contr.treatment(levels(island$Construction), base=which(levels(island$Construction) == 'interrogative'))
contrasts(island$Distance)     <- contr.treatment(levels(island$Distance), base=which(levels(island$Distance) == 'long'))
contrasts(island$Structure)    <- contr.treatment(levels(island$Structure), base=which(levels(island$Structure) == 'island'))

m_rev <- brm(formula = RespNum ~ Distance * Structure * Construction + Distance * Structure * c.(WMPartialCreditUnit)
             + (1 | Participant) 
             + (1 | Item),
             prior = priors,
             family = bernoulli(),
             data = island)

# Save models.
saveRDS(m, 'm_interrogative.rds')
saveRDS(m_rev, 'm_interrogative_rev.rds')


# Plot posterior distributions of interaction effects across islands (Figure 3) --------
# The code assigns the same area to each plot.
jpeg('Figure3.jpeg', units='cm', width=16, height=16, res=300)
grid.newpage()
grid.draw(rbind(cbind(ggplotGrob(plot_subject), ggplotGrob(plot_np)),
                cbind(ggplotGrob(plot_adjunct), ggplotGrob(plot_interrogative))))
dev.off()
                
ggplot_gtable(ggplot_build(x))
# Export data for public sharing ------------------------------------------
# Save to .csv file.
write.csv(island, file= 'islands_Spanish_data.csv', quote=F, row.names=F)  


